@@ -45,6 +45,10 @@
 #include "monitor.h"
 #endif
 
+#ifdef USE_AUTOCRYPT
+#include "autocrypt.h"
+#endif
+
 #include "mutt_crypt.h"
 
 
@@ -618,6 +622,7 @@ int mutt_index_menu (void)
   int do_buffy_notify = 1;
   int close = 0; /* did we OP_QUIT or OP_EXIT out of this menu? */
   int attach_msg = option(OPTATTACHMSG);
+  int in_pager = 0;  /* set when pager redirects a function through the index */
 
   menu = mutt_new_menu (MENU_MAIN);
   menu->make_entry = index_make_entry;
@@ -742,7 +747,7 @@ int mutt_index_menu (void)
     if (op >= 0)
       mutt_curs_set (0);
 
-    if (menu->menu == MENU_MAIN)
+    if (!in_pager)
     {
       index_menu_redraw (menu);
 
@@ -770,9 +775,8 @@ int mutt_index_menu (void)
 #if defined (USE_SLANG_CURSES) || defined (HAVE_RESIZETERM)
       if (SigWinch)
       {
-	mutt_flushinp ();
-	mutt_resize_screen ();
 	SigWinch = 0;
+	mutt_resize_screen ();
 	menu->top = 0; /* so we scroll the right amount */
 	/*
 	 * force a real complete redraw.  clrtobot() doesn't seem to be able
@@ -904,7 +908,7 @@ int mutt_index_menu (void)
 	if (mutt_get_field (_("Jump to message: "), buf, sizeof (buf), 0) != 0
 	    || !buf[0])
         {
-          if (menu->menu == MENU_PAGER)
+          if (in_pager)
           {
             op = OP_DISPLAY_MESSAGE;
             continue;
@@ -937,7 +941,7 @@ int mutt_index_menu (void)
 	  if (j >= 0)
 	  {
 	    menu->current = Context->hdrs[j]->virtual;
-	    if (menu->menu == MENU_PAGER)
+	    if (in_pager)
 	    {
 	      op = OP_DISPLAY_MESSAGE;
 	      continue;
@@ -1090,7 +1094,7 @@ int mutt_index_menu (void)
 	    resort_index (menu);
 	    set_option (OPTSEARCHINVALID);
 	  }
-	  if (menu->menu == MENU_PAGER)
+	  if (in_pager)
 	  {
 	    op = OP_DISPLAY_MESSAGE;
 	    continue;
@@ -1244,7 +1248,7 @@ int mutt_index_menu (void)
 	  FREE (&Context);
 
 	/* if we were in the pager, redisplay the message */
-	if (menu->menu == MENU_PAGER)
+	if (in_pager)
 	{
 	  op = OP_DISPLAY_MESSAGE;
 	  continue;
@@ -1261,13 +1265,20 @@ int mutt_index_menu (void)
 
 	if (attach_msg)
 	  op = OP_MAIN_CHANGE_FOLDER_READONLY;
+        /* fall through */
 
+      case OP_MAIN_BROWSE_MAILBOXES:
+        if (attach_msg && (op != OP_MAIN_CHANGE_FOLDER_READONLY))
+          op = OP_MAIN_BROWSE_MAILBOXES_READONLY;
+
+        /* fall through */
 	/* fallback to the readonly case */
 
+      case OP_MAIN_BROWSE_MAILBOXES_READONLY:
       case OP_MAIN_CHANGE_FOLDER_READONLY:
       {
         BUFFER *folderbuf;
-        int cont = 0;  /* Set if we want to continue instead of break */
+        int pager_return = 1;  /* return to display message in pager */
 
         folderbuf = mutt_buffer_pool_get ();
 
@@ -1289,13 +1300,13 @@ int mutt_index_menu (void)
 	}
 #ifdef USE_SIDEBAR
         else if (op == OP_SIDEBAR_OPEN)
-        {
-          const char *path = mutt_sb_get_highlight();
-          if (!path || !*path)
-            goto changefoldercleanup;
-          mutt_buffer_strcpy (folderbuf, path);
-        }
+          mutt_buffer_strcpy (folderbuf, NONULL (mutt_sb_get_highlight()));
 #endif
+
+        else if ((op == OP_MAIN_BROWSE_MAILBOXES) ||
+                 (op == OP_MAIN_BROWSE_MAILBOXES_READONLY))
+          mutt_buffer_select_file (folderbuf, MUTT_SEL_FOLDER | MUTT_SEL_BUFFY);
+
 	else
 	{
           if (option (OPTCHANGEFOLDERNEXT) && Context && Context->path)
@@ -1306,21 +1317,14 @@ int mutt_index_menu (void)
 	  mutt_buffer_buffy (folderbuf);
 
           if (mutt_buffer_enter_fname (cp, folderbuf, 1) == -1)
-          {
-            if (menu->menu == MENU_PAGER)
-            {
-              op = OP_DISPLAY_MESSAGE;
-              cont = 1;
-            }
             goto changefoldercleanup;
-          }
-	  if (!mutt_buffer_len (folderbuf))
-	  {
-            mutt_window_clearline (MuttMessageWindow, 0);
-	    goto changefoldercleanup;
-	  }
 	}
 
+        if (!mutt_buffer_len (folderbuf))
+        {
+          mutt_window_clearline (MuttMessageWindow, 0);
+          goto changefoldercleanup;
+        }
 	mutt_buffer_expand_path (folderbuf);
         if (mx_get_magic (mutt_b2s (folderbuf)) <= 0)
 	{
@@ -1328,6 +1332,9 @@ int mutt_index_menu (void)
 	  goto changefoldercleanup;
 	}
 
+        /* past this point, we don't return to the pager on error */
+        pager_return = 0;
+
 	/* keepalive failure in mutt_enter_fname may kill connection. #3028 */
 	if (Context && !Context->path)
 	  FREE (&Context);
@@ -1371,18 +1378,12 @@ int mutt_index_menu (void)
 
         mutt_sleep (0);
 
-        /* Note that menu->menu may be MENU_PAGER if the change folder
-         * operation originated from the pager.
-         *
-         * However, exec commands currently use CurrentMenu to determine what
-         * functions are available, which is automatically set by the
-         * mutt_push/pop_current_menu() functions.  If that changes, the menu
-         * would need to be reset here, and the pager cleanup code after the
-         * switch statement would need to be run. */
 	mutt_folder_hook (mutt_b2s (folderbuf));
 
 	if ((Context = mx_open_mailbox (mutt_b2s (folderbuf),
-					(option (OPTREADONLY) || op == OP_MAIN_CHANGE_FOLDER_READONLY) ?
+					(option (OPTREADONLY) ||
+                                         op == OP_MAIN_CHANGE_FOLDER_READONLY ||
+                                         op == OP_MAIN_BROWSE_MAILBOXES_READONLY) ?
 					MUTT_READONLY : 0, NULL)) != NULL)
 	{
 	  menu->current = ci_first_message ();
@@ -1406,10 +1407,12 @@ int mutt_index_menu (void)
 
       changefoldercleanup:
         mutt_buffer_pool_release (&folderbuf);
-        if (cont)
+        if (in_pager && pager_return)
+        {
+          op = OP_DISPLAY_MESSAGE;
           continue;
-        else
-          break;
+        }
+        break;
       }
 
       case OP_DISPLAY_MESSAGE:
@@ -1437,11 +1440,6 @@ int mutt_index_menu (void)
 	if (option (OPTPGPAUTODEC) && (tag || !(CURHDR->security & PGP_TRADITIONAL_CHECKED)))
 	  mutt_check_traditional_pgp (tag ? NULL : CURHDR, &menu->redraw);
 
-        /* If we are returning to the pager via an index menu redirection, we
-         * need to reset the menu->menu.  Otherwise mutt_pop_current_menu() will
-         * set CurrentMenu incorrectly when we return back to the index menu. */
-        menu->menu = MENU_MAIN;
-
         if ((op = mutt_display_message (CURHDR)) < 0)
 	{
 	  unset_option (OPTNEEDRESORT);
@@ -1449,22 +1447,22 @@ int mutt_index_menu (void)
 	}
 
         /* This is used to redirect a single operation back here afterwards.  If
-         * mutt_display_message() returns 0, then the menu and pager state will
+         * mutt_display_message() returns 0, then this flag and pager state will
          * be cleaned up after this switch statement. */
-	menu->menu = MENU_PAGER;
+	in_pager = 1;
  	menu->oldcurrent = menu->current;
 	continue;
 
       case OP_EXIT:
 
 	close = op;
-	if (menu->menu == MENU_MAIN && attach_msg)
+	if (!in_pager && attach_msg)
 	{
           done = 1;
           break;
 	}
 
-	if ((menu->menu == MENU_MAIN)
+	if ((!in_pager)
 	    && (query_quadoption (OPT_QUIT,
 				  _("Exit Mutt without saving?")) == MUTT_YES))
 	{
@@ -1498,7 +1496,7 @@ int mutt_index_menu (void)
 	  Context->changed = 1;
 	  mutt_message _("Thread broken");
 
-	  if (menu->menu == MENU_PAGER)
+	  if (in_pager)
 	  {
 	    op = OP_DISPLAY_MESSAGE;
 	    continue;
@@ -1542,7 +1540,7 @@ int mutt_index_menu (void)
 	    mutt_error _("No thread linked");
 	}
 
-	if (menu->menu == MENU_PAGER)
+	if (in_pager)
 	{
 	  op = OP_DISPLAY_MESSAGE;
 	  continue;
@@ -1559,7 +1557,7 @@ int mutt_index_menu (void)
 	CHECK_ATTACH;
 	mutt_edit_content_type (CURHDR, CURHDR->content, NULL);
 	/* if we were in the pager, redisplay the message */
-	if (menu->menu == MENU_PAGER)
+	if (in_pager)
 	{
 	  op = OP_DISPLAY_MESSAGE;
 	  continue;
@@ -1574,17 +1572,17 @@ int mutt_index_menu (void)
         CHECK_VISIBLE;
 	if (menu->current >= Context->vcount - 1)
 	{
-	  if (menu->menu == MENU_MAIN)
+	  if (!in_pager)
 	    mutt_error _("You are on the last message.");
 	  break;
 	}
 	if ((menu->current = ci_next_undeleted (menu->current)) == -1)
 	{
 	  menu->current = menu->oldcurrent;
-	  if (menu->menu == MENU_MAIN)
+	  if (!in_pager)
 	    mutt_error _("No undeleted messages.");
 	}
-	else if (menu->menu == MENU_PAGER)
+	else if (in_pager)
 	{
 	  op = OP_DISPLAY_MESSAGE;
 	  continue;
@@ -1599,12 +1597,12 @@ int mutt_index_menu (void)
         CHECK_VISIBLE;
 	if (menu->current >= Context->vcount - 1)
 	{
-	  if (menu->menu == MENU_MAIN)
+	  if (!in_pager)
 	    mutt_error _("You are on the last message.");
 	  break;
 	}
 	menu->current++;
-	if (menu->menu == MENU_PAGER)
+	if (in_pager)
 	{
 	  op = OP_DISPLAY_MESSAGE;
 	  continue;
@@ -1625,10 +1623,10 @@ int mutt_index_menu (void)
 	if ((menu->current = ci_previous_undeleted (menu->current)) == -1)
 	{
 	  menu->current = menu->oldcurrent;
-	  if (menu->menu == MENU_MAIN)
+	  if (!in_pager)
 	    mutt_error _("No undeleted messages.");
 	}
-	else if (menu->menu == MENU_PAGER)
+	else if (in_pager)
 	{
 	  op = OP_DISPLAY_MESSAGE;
 	  continue;
@@ -1643,11 +1641,11 @@ int mutt_index_menu (void)
         CHECK_VISIBLE;
 	if (menu->current < 1)
 	{
-	  if (menu->menu == MENU_MAIN) mutt_error _("You are on the first message.");
+	  if (!in_pager) mutt_error _("You are on the first message.");
 	  break;
 	}
 	menu->current--;
-	if (menu->menu == MENU_PAGER)
+	if (in_pager)
 	{
 	  op = OP_DISPLAY_MESSAGE;
 	  continue;
@@ -1783,7 +1781,7 @@ int mutt_index_menu (void)
               mutt_error (_("No unread messages."));
           }
 	}
-	else if (menu->menu == MENU_PAGER)
+	else if (in_pager)
 	{
 	  op = OP_DISPLAY_MESSAGE;
 	  continue;
@@ -1881,7 +1879,7 @@ int mutt_index_menu (void)
 	CHECK_IN_MAILBOX;
 	if (mx_toggle_write (Context) == 0)
         {
-	  if (menu->menu == MENU_PAGER)
+	  if (in_pager)
           {
             op = OP_DISPLAY_MESSAGE;
             continue;
@@ -1925,7 +1923,7 @@ int mutt_index_menu (void)
 	  else
 	    mutt_error _("You are on the first thread.");
 	}
-	else if (menu->menu == MENU_PAGER)
+	else if (in_pager)
 	{
 	  op = OP_DISPLAY_MESSAGE;
 	  continue;
@@ -1945,7 +1943,7 @@ int mutt_index_menu (void)
 	{
 	  menu->current = menu->oldcurrent;
 	}
-	else if (menu->menu == MENU_PAGER)
+	else if (in_pager)
         {
           op = OP_DISPLAY_MESSAGE;
           continue;
@@ -2132,7 +2130,7 @@ int mutt_index_menu (void)
 	      menu->current = menu->oldcurrent;
 	      menu->redraw |= REDRAW_CURRENT;
 	    }
-	    else if (menu->menu == MENU_PAGER)
+	    else if (in_pager)
 	    {
 	      op = OP_DISPLAY_MESSAGE;
 	      continue;
@@ -2271,7 +2269,7 @@ int mutt_index_menu (void)
         if (tag || !(CURHDR->security & PGP_TRADITIONAL_CHECKED))
 	  mutt_check_traditional_pgp (tag ? NULL : CURHDR, &menu->redraw);
 
-        if (menu->menu == MENU_PAGER)
+        if (in_pager)
         {
 	  op = OP_DISPLAY_MESSAGE;
 	  continue;
@@ -2333,7 +2331,7 @@ int mutt_index_menu (void)
 	    if ((menu->current = (op == OP_MAIN_READ_THREAD ?
 				  mutt_next_thread (CURHDR) : mutt_next_subthread (CURHDR))) == -1)
 	      menu->current = menu->oldcurrent;
-	    else if (menu->menu == MENU_PAGER)
+	    else if (in_pager)
 	    {
 	      op = OP_DISPLAY_MESSAGE;
 	      continue;
@@ -2360,7 +2358,7 @@ int mutt_index_menu (void)
 	  if (!mutt_get_field (_("Enter macro stroke: "), buf, sizeof(buf),
 	  		       MUTT_CLEAR) && buf[0])
 	  {
-	    snprintf(str, sizeof(str), "%s%s", MarkMacroPrefix, buf);
+	    snprintf(str, sizeof(str), "%s%s", NONULL (MarkMacroPrefix), buf);
 	    snprintf(macro, sizeof(macro),
 		     "<search>~i \"%s\"\n", CURHDR->env->message_id);
             /* L10N: "message hotkey" is the key bindings menu description of a
@@ -2562,15 +2560,22 @@ int mutt_index_menu (void)
         mutt_reflow_windows();
 	break;
 #endif
+
+#ifdef USE_AUTOCRYPT
+      case OP_AUTOCRYPT_ACCT_MENU:
+        mutt_autocrypt_account_menu ();
+        break;
+#endif
+
       default:
-	if (menu->menu == MENU_MAIN)
+	if (!in_pager)
 	  km_error_key (MENU_MAIN);
     }
 
-    if (menu->menu == MENU_PAGER)
+    if (in_pager)
     {
       mutt_clear_pager_position ();
-      menu->menu = MENU_MAIN;
+      in_pager = 0;
       menu->redraw = REDRAW_FULL;
     }
 
