@@ -52,6 +52,9 @@
 # ifndef HAVE_LIBIDN
 #  define HAVE_LIBIDN
 # endif
+# ifndef HAVE_LIBIDN2
+#  define HAVE_LIBIDN2
+# endif
 # ifndef HAVE_GETADDRINFO
 #  define HAVE_GETADDRINFO
 # endif
@@ -64,4 +67,7 @@
 # ifndef USE_COMPRESSED
 #  define USE_COMPRESSED
 # endif
+# ifndef USE_AUTOCRYPT
+#  define USE_AUTOCRYPT
+# endif
 #endif
