@@ -47,7 +47,7 @@ group_t *mutt_pattern_group (const char *k)
     dprint (2, (debugfile, "mutt_pattern_group: Creating group %s.\n", k));
     p = safe_calloc (1, sizeof (group_t));
     p->name = safe_strdup (k);
-    hash_insert (Groups, p->name, p, 0);
+    hash_insert (Groups, p->name, p);
   }
 
   return p;
@@ -205,4 +205,3 @@ int mutt_group_match (group_t *g, const char *s)
   }
   return 0;
 }
-
