@@ -130,41 +130,52 @@ static void imap_msn_index_to_uid_seqset (BUFFER *b, IMAP_DATA *idata)
   }
 }
 
-static int imap_hcache_namer (const char* path, char* dest, size_t dlen)
+static void imap_hcache_namer (const char *path, BUFFER *dest)
 {
-  return snprintf (dest, dlen, "%s.hcache", path);
+  mutt_buffer_printf (dest, "%s.hcache", path);
 }
 
 header_cache_t* imap_hcache_open (IMAP_DATA* idata, const char* path)
 {
   IMAP_MBOX mx;
   ciss_url_t url;
-  char cachepath[LONG_STRING];
-  char mbox[LONG_STRING];
+  BUFFER *cachepath = NULL;
+  BUFFER *mbox = NULL;
   size_t len;
+  header_cache_t *rv = NULL;
+
+  mbox = mutt_buffer_pool_get ();
+  cachepath = mutt_buffer_pool_get ();
 
   if (path)
-    imap_cachepath (idata, path, mbox, sizeof (mbox));
+    imap_cachepath (idata, path, mbox);
   else
   {
     if (!idata->ctx || imap_parse_path (idata->ctx->path, &mx) < 0)
-      return NULL;
+      goto cleanup;
 
-    imap_cachepath (idata, mx.mbox, mbox, sizeof (mbox));
+    imap_cachepath (idata, mx.mbox, mbox);
     FREE (&mx.mbox);
   }
 
-  if (strstr(mbox, "/../") || (strcmp(mbox, "..") == 0) || (strncmp(mbox, "../", 3) == 0))
-    return NULL;
-  len = strlen(mbox);
-  if ((len > 3) && (strcmp(mbox + len - 3, "/..") == 0))
-    return NULL;
+  if (strstr (mutt_b2s (mbox), "/../") ||
+      (strcmp (mutt_b2s (mbox), "..") == 0) ||
+      (strncmp(mutt_b2s (mbox), "../", 3) == 0))
+    goto cleanup;
+  len = mutt_buffer_len (mbox);
+  if ((len > 3) && (strcmp(mutt_b2s (mbox) + len - 3, "/..") == 0))
+    goto cleanup;
 
   mutt_account_tourl (&idata->conn->account, &url);
-  url.path = mbox;
-  url_ciss_tostring (&url, cachepath, sizeof (cachepath), U_PATH);
+  url.path = mbox->data;
+  url_ciss_tobuffer (&url, cachepath, U_PATH);
 
-  return mutt_hcache_open (HeaderCache, cachepath, imap_hcache_namer);
+  rv = mutt_hcache_open (HeaderCache, mutt_b2s (cachepath), imap_hcache_namer);
+
+cleanup:
+  mutt_buffer_pool_release (&mbox);
+  mutt_buffer_pool_release (&cachepath);
+  return rv;
 }
 
 void imap_hcache_close (IMAP_DATA* idata)
@@ -179,22 +190,24 @@ void imap_hcache_close (IMAP_DATA* idata)
 HEADER* imap_hcache_get (IMAP_DATA* idata, unsigned int uid)
 {
   char key[16];
-  unsigned int* uv;
+  void *data;
+  unsigned int uv;
   HEADER* h = NULL;
 
   if (!idata->hcache)
     return NULL;
 
   sprintf (key, "/%u", uid);
-  uv = (unsigned int*)mutt_hcache_fetch (idata->hcache, key,
-                                         imap_hcache_keylen);
-  if (uv)
+  data = mutt_hcache_fetch (idata->hcache, key,
+                            imap_hcache_keylen);
+  if (data)
   {
-    if (*uv == idata->uid_validity)
-      h = mutt_hcache_restore ((unsigned char*)uv, NULL);
+    memcpy (&uv, data, sizeof(unsigned int));
+    if (uv == idata->uid_validity)
+      h = mutt_hcache_restore ((unsigned char *)data, NULL);
     else
-      dprint (3, (debugfile, "hcache uidvalidity mismatch: %u", *uv));
-    mutt_hcache_free ((void **)&uv);
+      dprint (3, (debugfile, "hcache uidvalidity mismatch: %u", uv));
+    mutt_hcache_free ((void **)&data);
   }
 
   return h;
@@ -546,30 +559,27 @@ char *imap_fix_path (IMAP_DATA *idata, const char *mailbox, char *path,
   return path;
 }
 
-void imap_cachepath(IMAP_DATA* idata, const char* mailbox, char* dest,
-                    size_t dlen)
+void imap_cachepath (IMAP_DATA *idata, const char *mailbox, BUFFER *dest)
 {
-  char* s;
-  const char* p = mailbox;
+  const char *p = mailbox;
 
-  for (s = dest; p && *p && dlen; dlen--)
+  mutt_buffer_clear (dest);
+  if (!p)
+    return;
+
+  while (*p)
   {
     if (*p == idata->delim)
     {
-      *s = '/';
+      mutt_buffer_addch (dest, '/');
       /* simple way to avoid collisions with UIDs */
       if (*(p + 1) >= '0' && *(p + 1) <= '9')
-      {
-	if (--dlen)
-	  *++s = '_';
-      }
+        mutt_buffer_addch (dest, '_');
     }
     else
-      *s = *p;
+      mutt_buffer_addch (dest, *p);
     p++;
-    s++;
   }
-  *s = '\0';
 }
 
 /* imap_get_literal_count: write number of bytes in an IMAP literal into
@@ -707,6 +717,16 @@ void imap_qualify_path (char *dest, size_t len, IMAP_MBOX *mx, char* path)
   url_ciss_tostring (&url, dest, len, 0);
 }
 
+void imap_buffer_qualify_path (BUFFER *dest, IMAP_MBOX *mx, char* path)
+{
+  ciss_url_t url;
+
+  mutt_account_tourl (&mx->account, &url);
+  url.path = path;
+
+  url_ciss_tobuffer (&url, dest, 0);
+}
+
 
 static void _imap_quote_string (char *dest, size_t dlen, const char *src,
                                 const char *to_quote)
