@@ -789,6 +789,8 @@ void mutt_pop_current_menu (MUTTMENU *menu)
   else
   {
     CurrentMenu = MENU_MAIN;
+    move (0, 0);
+    clrtobot ();
   }
 }
 
@@ -1073,8 +1075,8 @@ int mutt_menuLoop (MUTTMENU *menu)
 #if defined (USE_SLANG_CURSES) || defined (HAVE_RESIZETERM)
     if (SigWinch)
     {
-      mutt_resize_screen ();
       SigWinch = 0;
+      mutt_resize_screen ();
       clearok(stdscr,TRUE);/*force complete redraw*/
     }
 #endif
