@@ -515,6 +515,8 @@ enum
   OPTSIDEBARFOLDERINDENT,
   OPTSIDEBARNEWMAILONLY,
   OPTSIDEBARNEXTNEWWRAP,
+  OPTSIDEBARRELSPINDENT,
+  OPTSIDEBARUSEMBSHORTCUTS,
   OPTSIDEBARSHORTPATH,
 #endif
   OPTSIGDASHES,
