@@ -1,20 +1,20 @@
 /*
  * Copyright (C) 2003,2005,2008-2009 Thomas Roessler <roessler@does-not-exist.org>
- * 
+ *
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
  *     the Free Software Foundation; either version 2 of the License, or
  *     (at your option) any later version.
- * 
+ *
  *     This program is distributed in the hope that it will be useful,
  *     but WITHOUT ANY WARRANTY; without even the implied warranty of
  *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *     GNU General Public License for more details.
- * 
+ *
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
  *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
- */ 
+ */
 
 #if HAVE_CONFIG_H
 # include "config.h"
@@ -24,7 +24,7 @@
 #include "charset.h"
 #include "mutt_idna.h"
 
-#ifdef HAVE_LIBIDN
+#if defined(HAVE_LIBIDN) || defined(HAVE_LIBIDN2)
 static int check_idn (char *domain)
 {
   if (! domain)
@@ -41,7 +41,7 @@ static int check_idn (char *domain)
 
   return 0;
 }
-#endif /* HAVE_LIBIDN */
+#endif /* defined(HAVE_LIBIDN) || defined(HAVE_LIBIDN2) */
 
 static int mbox_to_udomain (const char *mbx, char **user, char **domain)
 {
@@ -90,14 +90,14 @@ static char *intl_to_local (char *orig_user, char *orig_domain, int flags)
   char *local_user = NULL, *local_domain = NULL, *mailbox = NULL;
   char *reversed_user = NULL, *reversed_domain = NULL;
   char *tmp = NULL;
-#ifdef HAVE_LIBIDN
+#if defined(HAVE_LIBIDN) || defined(HAVE_LIBIDN2)
   int is_idn_encoded = 0;
-#endif /* HAVE_LIBIDN */
+#endif /* defined(HAVE_LIBIDN) || defined(HAVE_LIBIDN2) */
 
   local_user = safe_strdup (orig_user);
   local_domain = safe_strdup (orig_domain);
 
-#ifdef HAVE_LIBIDN
+#if defined(HAVE_LIBIDN) || defined(HAVE_LIBIDN2)
   is_idn_encoded = check_idn (local_domain);
   if (is_idn_encoded && option (OPTIDNDECODE))
   {
@@ -106,7 +106,7 @@ static char *intl_to_local (char *orig_user, char *orig_domain, int flags)
     mutt_str_replace (&local_domain, tmp);
     FREE (&tmp);
   }
-#endif /* HAVE_LIBIDN */
+#endif /* defined(HAVE_LIBIDN) || defined(HAVE_LIBIDN2) */
 
   /* we don't want charset-hook effects, so we set flags to 0 */
   if (mutt_convert_string (&local_user, "utf-8", Charset, 0) == -1)
@@ -148,7 +148,7 @@ static char *intl_to_local (char *orig_user, char *orig_domain, int flags)
       goto cleanup;
     }
 
-#ifdef HAVE_LIBIDN
+#if defined(HAVE_LIBIDN) || defined(HAVE_LIBIDN2)
     /* If the original domain was UTF-8, idna encoding here could
      * produce a non-matching domain!  Thus we only want to do the
      * idna_to_ascii_8z() if the original domain was IDNA encoded.
@@ -164,7 +164,7 @@ static char *intl_to_local (char *orig_user, char *orig_domain, int flags)
       }
       mutt_str_replace (&reversed_domain, tmp);
     }
-#endif /* HAVE_LIBIDN */
+#endif /* defined(HAVE_LIBIDN) || defined(HAVE_LIBIDN2) */
 
     if (ascii_strcasecmp (orig_domain, reversed_domain))
     {
@@ -203,14 +203,14 @@ static char *local_to_intl (char *user, char *domain)
   if (mutt_convert_string (&intl_domain, Charset, "utf-8", 0) == -1)
     goto cleanup;
 
-#ifdef HAVE_LIBIDN
+#if defined(HAVE_LIBIDN) || defined(HAVE_LIBIDN2)
   if (option (OPTIDNENCODE))
   {
     if (idna_to_ascii_8z (intl_domain, &tmp, IDNA_ALLOW_UNASSIGNED) != IDNA_SUCCESS)
       goto cleanup;
     mutt_str_replace (&intl_domain, tmp);
   }
-#endif /* HAVE_LIBIDN */
+#endif /* defined(HAVE_LIBIDN) || defined(HAVE_LIBIDN2) */
 
   mailbox = safe_malloc (mutt_strlen (intl_user) + mutt_strlen (intl_domain) + 2);
   sprintf (mailbox, "%s@%s", NONULL(intl_user), NONULL(intl_domain)); /* __SPRINTF_CHECKED__ */
@@ -320,10 +320,10 @@ void mutt_env_to_local (ENVELOPE *e)
  * by preprocessor at the macro-expansion time.
  * Note that #a escapes and double quotes the argument.
  */
-#define H_TO_INTL(a)	\
-  if (mutt_addrlist_to_intl (env->a, err) && !e) \
-  { \
-     if (tag) *tag = #a; e = 1; err = NULL; \
+#define H_TO_INTL(a)                                    \
+  if (mutt_addrlist_to_intl (env->a, err) && !e)        \
+  {                                                     \
+    if (tag) *tag = #a; e = 1; err = NULL;              \
   }
 
 int mutt_env_to_intl (ENVELOPE *env, char **tag, char **err)
