@@ -58,18 +58,19 @@ typedef struct menu_t
   mutt_window_t *helpwin;
   mutt_window_t *messagewin;
 
-  /* Setting dialog != NULL overrides normal menu behavior. 
+  /* Setting dialog != NULL overrides normal menu behavior.
    * In dialog mode menubar is hidden and prompt keys are checked before
-   * normal menu movement keys. This can cause problems with scrolling, if 
+   * normal menu movement keys. This can cause problems with scrolling, if
    * prompt keys override movement keys.
    */
   char **dialog;	/* dialog lines themselves */
+  int dsize;            /* number of allocated dialog lines */
   char *prompt;		/* prompt for user, similar to mutt_multi_choice */
   char *keys;		/* keys used in the prompt */
-  
+
   /* callback to generate an index line for the requested element */
   void (*make_entry) (char *, size_t, struct menu_t *, int);
-  
+
   /* how to search the menu */
   int (*search) (struct menu_t *, regex_t *re, int n);
 
@@ -79,11 +80,11 @@ typedef struct menu_t
   void (*custom_menu_redraw) (struct menu_t *);
   void *redraw_data;
 
-  /* color pair to be used for the requested element 
+  /* color pair to be used for the requested element
    * (default function returns ColorDefs[MT_COLOR_NORMAL])
    */
   int (*color) (int i);
-   
+
   /* the following are used only by mutt_menuLoop() */
   int top;		/* entry that is the top of the current page */
   int oldcurrent;	/* for driver use only. */
@@ -124,6 +125,7 @@ void mutt_ts_icon (char *);
 
 MUTTMENU *mutt_new_menu (int);
 void mutt_menuDestroy (MUTTMENU **);
+void mutt_menu_add_dialog_row (MUTTMENU *, const char *);
 void mutt_push_current_menu (MUTTMENU *);
 void mutt_pop_current_menu (MUTTMENU *);
 void mutt_set_current_menu_redraw (int);
