@@ -146,7 +146,7 @@ int mutt_which_case (const char *s)
 static int
 msg_search (CONTEXT *ctx, pattern_t* pat, int msgno)
 {
-  char tempfile[_POSIX_PATH_MAX];
+  BUFFER *tempfile = NULL;
   MESSAGE *msg = NULL;
   STATE s;
   struct stat st;
@@ -165,11 +165,13 @@ msg_search (CONTEXT *ctx, pattern_t* pat, int msgno)
       memset (&s, 0, sizeof (s));
       s.fpin = msg->fp;
       s.flags = MUTT_CHARCONV;
-      mutt_mktemp (tempfile, sizeof (tempfile));
-      if ((s.fpout = safe_fopen (tempfile, "w+")) == NULL)
+
+      tempfile = mutt_buffer_new ();
+      mutt_buffer_mktemp (tempfile);
+      if ((s.fpout = safe_fopen (mutt_b2s (tempfile), "w+")) == NULL)
       {
-	mutt_perror (tempfile);
-	return (0);
+	mutt_perror (mutt_b2s (tempfile));
+	goto cleanup;
       }
 
       if (pat->op != MUTT_BODY)
@@ -186,9 +188,9 @@ msg_search (CONTEXT *ctx, pattern_t* pat, int msgno)
 	  if (s.fpout)
 	  {
 	    safe_fclose (&s.fpout);
-	    unlink (tempfile);
+	    unlink (mutt_b2s (tempfile));
 	  }
-	  return (0);
+	  goto cleanup;
 	}
 
 	fseeko (msg->fp, h->offset, 0);
@@ -246,10 +248,12 @@ msg_search (CONTEXT *ctx, pattern_t* pat, int msgno)
     if (option (OPTTHOROUGHSRC))
     {
       safe_fclose (&fp);
-      unlink (tempfile);
+      unlink (mutt_b2s (tempfile));
     }
   }
 
+cleanup:
+  mutt_buffer_free (&tempfile);
   return match;
 }
 
@@ -1497,13 +1501,12 @@ int mutt_pattern_func (int op, char *prompt)
   buf = mutt_buffer_pool_get ();
 
   mutt_buffer_strcpy (buf, NONULL (Context->pattern));
-  if (mutt_get_field (prompt, buf->data, buf->dsize, MUTT_PATTERN | MUTT_CLEAR) != 0 ||
-      !(mutt_b2s (buf)[0]))
+  if ((mutt_buffer_get_field (prompt, buf, MUTT_PATTERN | MUTT_CLEAR) != 0) ||
+      !mutt_buffer_len (buf))
   {
     mutt_buffer_pool_release (&buf);
     return (-1);
   }
-  mutt_buffer_fix_dptr (buf);
 
   mutt_message _("Compiling search pattern...");
 
@@ -1568,6 +1571,7 @@ int mutt_pattern_func (int op, char *prompt)
           case MUTT_UNDELETE:
             mutt_set_flag (Context, Context->hdrs[Context->v2r[i]], MUTT_PURGE,
                            0);
+            /* fall through */
 	  case MUTT_DELETE:
 	    mutt_set_flag (Context, Context->hdrs[Context->v2r[i]], MUTT_DELETE,
                            (op == MUTT_DELETE));
