@@ -408,7 +408,7 @@ static void attach_forward_bodies (FILE * fp, HEADER * hdr,
   FILE *parent_fp;
   HEADER *tmphdr = NULL;
   BODY **last;
-  char tmpbody[_POSIX_PATH_MAX];
+  BUFFER *tmpbody = NULL;
   FILE *tmpfp = NULL;
 
   char prefix[STRING];
@@ -441,11 +441,12 @@ static void attach_forward_bodies (FILE * fp, HEADER * hdr,
   tmphdr->env = mutt_new_envelope ();
   mutt_make_forward_subject (tmphdr->env, Context, parent_hdr);
 
-  mutt_mktemp (tmpbody, sizeof (tmpbody));
-  if ((tmpfp = safe_fopen (tmpbody, "w")) == NULL)
+  tmpbody = mutt_buffer_pool_get ();
+  mutt_buffer_mktemp (tmpbody);
+  if ((tmpfp = safe_fopen (mutt_b2s (tmpbody), "w")) == NULL)
   {
-    mutt_error (_("Can't open temporary file %s."), tmpbody);
-    return;
+    mutt_error (_("Can't open temporary file %s."), mutt_b2s (tmpbody));
+    goto bail;
   }
 
   mutt_forward_intro (Context, parent_hdr, tmpfp);
@@ -554,16 +555,18 @@ static void attach_forward_bodies (FILE * fp, HEADER * hdr,
   tmpfp = NULL;
 
   /* now that we have the template, send it. */
-  ci_send_message (0, tmphdr, tmpbody, NULL, parent_hdr);
+  ci_send_message (0, tmphdr, mutt_b2s (tmpbody), NULL, parent_hdr);
+
+  mutt_buffer_pool_release (&tmpbody);
   return;
 
 bail:
-
   if (tmpfp)
   {
     safe_fclose (&tmpfp);
-    mutt_unlink (tmpbody);
+    mutt_unlink (mutt_b2s (tmpbody));
   }
+  mutt_buffer_pool_release (&tmpbody);
 
   mutt_free_header (&tmphdr);
 }
@@ -584,12 +587,12 @@ static void attach_forward_msgs (FILE * fp, HEADER * hdr,
                                  ATTACH_CONTEXT *actx, BODY * cur)
 {
   HEADER *curhdr = NULL;
-  HEADER *tmphdr;
+  HEADER *tmphdr = NULL;
   short i;
   int rc;
 
   BODY **last;
-  char tmpbody[_POSIX_PATH_MAX];
+  BUFFER *tmpbody = NULL;
   FILE *tmpfp = NULL;
 
   int cmflags = 0;
@@ -612,7 +615,7 @@ static void attach_forward_msgs (FILE * fp, HEADER * hdr,
   mutt_make_forward_subject (tmphdr->env, Context, curhdr);
 
 
-  tmpbody[0] = '\0';
+  tmpbody = mutt_buffer_pool_get ();
 
   if ((rc = query_quadoption (OPT_MIMEFWD,
                               _("Forward MIME encapsulated?"))) == MUTT_NO)
@@ -620,12 +623,11 @@ static void attach_forward_msgs (FILE * fp, HEADER * hdr,
 
     /* no MIME encapsulation */
 
-    mutt_mktemp (tmpbody, sizeof (tmpbody));
-    if (!(tmpfp = safe_fopen (tmpbody, "w")))
+    mutt_buffer_mktemp (tmpbody);
+    if (!(tmpfp = safe_fopen (mutt_b2s (tmpbody), "w")))
     {
-      mutt_error (_("Can't create %s."), tmpbody);
-      mutt_free_header (&tmphdr);
-      return;
+      mutt_error (_("Can't create %s."), mutt_b2s (tmpbody));
+      goto cleanup;
     }
 
     if (option (OPTFORWQUOTE))
@@ -686,9 +688,14 @@ static void attach_forward_msgs (FILE * fp, HEADER * hdr,
   else
     mutt_free_header (&tmphdr);
 
-  ci_send_message (0, tmphdr, *tmpbody ? tmpbody : NULL,
+  ci_send_message (0, tmphdr,
+                   mutt_buffer_len (tmpbody) ? mutt_b2s (tmpbody) : NULL,
 		   NULL, curhdr);
+  tmphdr = NULL;  /* ci_send_message frees this */
 
+cleanup:
+  mutt_free_header (&tmphdr);
+  mutt_buffer_pool_release (&tmpbody);
 }
 
 void mutt_attach_forward (FILE * fp, HEADER * hdr,
@@ -873,8 +880,8 @@ void mutt_attach_reply (FILE * fp, HEADER * hdr,
   short i;
 
   STATE st;
-  char tmpbody[_POSIX_PATH_MAX];
-  FILE *tmpfp;
+  BUFFER *tmpbody = NULL;
+  FILE *tmpfp = NULL;
 
   char prefix[SHORT_STRING];
   int rc;
@@ -910,17 +917,14 @@ void mutt_attach_reply (FILE * fp, HEADER * hdr,
 
   if (attach_reply_envelope_defaults (tmphdr->env, actx,
 				      parent_hdr ? parent_hdr : (cur ? cur->hdr : NULL), flags) == -1)
-  {
-    mutt_free_header (&tmphdr);
-    return;
-  }
+    goto cleanup;
 
-  mutt_mktemp (tmpbody, sizeof (tmpbody));
-  if ((tmpfp = safe_fopen (tmpbody, "w")) == NULL)
+  tmpbody = mutt_buffer_pool_get ();
+  mutt_buffer_mktemp (tmpbody);
+  if ((tmpfp = safe_fopen (mutt_b2s (tmpbody), "w")) == NULL)
   {
-    mutt_error (_("Can't create %s."), tmpbody);
-    mutt_free_header (&tmphdr);
-    return;
+    mutt_error (_("Can't create %s."), mutt_b2s (tmpbody));
+    goto cleanup;
   }
 
   if (!parent_hdr)
@@ -987,15 +991,24 @@ void mutt_attach_reply (FILE * fp, HEADER * hdr,
     if (mime_reply_any && !cur &&
 	copy_problematic_attachments (&tmphdr->content, actx, 0) == NULL)
     {
-      mutt_free_header (&tmphdr);
-      safe_fclose (&tmpfp);
-      return;
+      goto cleanup;
     }
   }
 
   safe_fclose (&tmpfp);
 
-  if (ci_send_message (flags, tmphdr, tmpbody, NULL,
+  if (ci_send_message (flags, tmphdr, mutt_b2s (tmpbody), NULL,
                        parent_hdr ? parent_hdr : (cur ? cur->hdr : NULL)) == 0)
     mutt_set_flag (Context, hdr, MUTT_REPLIED, 1);
+
+  tmphdr = NULL;  /* ci_send_message frees this */
+
+cleanup:
+  if (tmpfp)
+  {
+    safe_fclose (&tmpfp);
+    mutt_unlink (mutt_b2s (tmpbody));
+  }
+  mutt_buffer_pool_release (&tmpbody);
+  mutt_free_header (&tmphdr);
 }
