@@ -1,23 +1,23 @@
 /*
  * Copyright (C) 1999-2001 Thomas Roessler <roessler@does-not-exist.org>
- * 
+ *
  *     This program is free software; you can redistribute it
  *     and/or modify it under the terms of the GNU General Public
  *     License as published by the Free Software Foundation; either
  *     version 2 of the License, or (at your option) any later
  *     version.
- * 
+ *
  *     This program is distributed in the hope that it will be
  *     useful, but WITHOUT ANY WARRANTY; without even the implied
  *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
  *     PURPOSE.  See the GNU General Public License for more
  *     details.
- * 
+ *
  *     You should have received a copy of the GNU General Public
  *     License along with this program; if not, write to the Free
  *     Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  *     Boston, MA  02110-1301, USA.
- */ 
+ */
 
 /*
  * Mixmaster support for Mutt
@@ -75,11 +75,11 @@ static int mix_get_caps (const char *capstr)
       case 'C':
       	caps |= MIX_CAP_COMPRESS;
         break;
-      
+
       case 'M':
         caps |= MIX_CAP_MIDDLEMAN;
         break;
-      
+
       case 'N':
       {
 	switch (*++capstr)
@@ -87,18 +87,18 @@ static int mix_get_caps (const char *capstr)
 	  case 'm':
 	    caps |= MIX_CAP_NEWSMAIL;
 	    break;
-	  
+
 	  case 'p':
 	    caps |= MIX_CAP_NEWSPOST;
 	    break;
-	  
+
 	}
       }
     }
-    
+
     if (*capstr) capstr++;
   }
-  
+
   return caps;
 }
 
@@ -110,7 +110,7 @@ static void mix_add_entry (REMAILER ***type2_list, REMAILER *entry,
     *slots += 5;
     safe_realloc (type2_list, sizeof (REMAILER *) * (*slots));
   }
-  
+
   (*type2_list)[(*used)++] = entry;
   if (entry) entry->num = *used;
 }
@@ -125,7 +125,7 @@ static void mix_free_remailer (REMAILER **r)
   FREE (&(*r)->shortname);
   FREE (&(*r)->addr);
   FREE (&(*r)->ver);
-  
+
   FREE (r);		/* __FREE_CHECKED__ */
 }
 
@@ -137,47 +137,52 @@ static REMAILER **mix_type2_list (size_t *l)
   pid_t mm_pid;
   int devnull;
 
-  char cmd[HUGE_STRING + _POSIX_PATH_MAX];
+  BUFFER *cmd = NULL;
   char line[HUGE_STRING];
   char *t;
-  
+
   REMAILER **type2_list = NULL, *p;
   size_t slots = 0, used = 0;
 
   if (!l)
     return NULL;
-  
+
   if ((devnull = open ("/dev/null", O_RDWR)) == -1)
     return NULL;
-  
-  snprintf (cmd, sizeof (cmd), "%s -T", Mixmaster);
-  
-  if ((mm_pid = mutt_create_filter_fd (cmd, NULL, &fp, NULL, devnull, -1, devnull)) == -1)
+
+  cmd = mutt_buffer_pool_get ();
+  mutt_buffer_printf (cmd, "%s -T", Mixmaster);
+
+  if ((mm_pid = mutt_create_filter_fd (mutt_b2s (cmd), NULL, &fp, NULL, devnull,
+                                       -1, devnull)) == -1)
   {
+    mutt_buffer_pool_release (&cmd);
     close (devnull);
     return NULL;
   }
 
+  mutt_buffer_pool_release (&cmd);
+
   /* first, generate the "random" remailer */
-  
+
   p = mix_new_remailer ();
   p->shortname = safe_strdup ("<random>");
   mix_add_entry (&type2_list, p, &slots, &used);
-  
+
   while (fgets (line, sizeof (line), fp))
   {
     p = mix_new_remailer ();
-    
+
     if (!(t = strtok (line, " \t\n")))
       goto problem;
-    
+
     p->shortname = safe_strdup (t);
-    
+
     if (!(t = strtok (NULL, " \t\n")))
       goto problem;
 
     p->addr = safe_strdup (t);
-    
+
     if (!(t = strtok (NULL, " \t\n")))
       goto problem;
 
@@ -185,26 +190,26 @@ static REMAILER **mix_type2_list (size_t *l)
       goto problem;
 
     p->ver = safe_strdup (t);
-    
+
     if (!(t = strtok (NULL, " \t\n")))
       goto problem;
 
     p->caps = mix_get_caps (t);
-    
+
     mix_add_entry (&type2_list, p, &slots, &used);
     continue;
-    
-    problem:
+
+  problem:
     mix_free_remailer (&p);
   }
-  
+
   *l = used;
 
   mix_add_entry (&type2_list, NULL, &slots, &used);
   mutt_wait_filter (mm_pid);
 
   close (devnull);
-  
+
   return type2_list;
 }
 
@@ -212,10 +217,10 @@ static void mix_free_type2_list (REMAILER ***ttlp)
 {
   int i;
   REMAILER **type2_list = *ttlp;
-  
+
   for (i = 0; type2_list[i]; i++)
     mix_free_remailer (&type2_list[i]);
-  
+
   FREE (type2_list);		/* __FREE_CHECKED__ */
 }
 
@@ -235,11 +240,11 @@ static void mix_screen_coordinates (REMAILER **type2_list,
 
   if (!chain->cl)
     return;
-  
+
   safe_realloc (coordsp, sizeof (struct coord) * chain->cl);
-  
+
   coords = *coordsp;
-  
+
   if (i)
   {
     c = coords[i-1].c + strlen (type2_list[chain->ch[i-1]]->shortname) + 2;
@@ -250,8 +255,8 @@ static void mix_screen_coordinates (REMAILER **type2_list,
     r = MIX_VOFFSET;
     c = MIX_HOFFSET;
   }
-    
-  
+
+
   for (; i < chain->cl; i++)
   {
     oc = c;
@@ -262,12 +267,12 @@ static void mix_screen_coordinates (REMAILER **type2_list,
       oc = c = MIX_HOFFSET;
       r++;
     }
-    
+
     coords[i].c = oc;
     coords[i].r = r;
-    
+
   }
-  
+
 }
 
 static void mix_redraw_ce (REMAILER **type2_list,
@@ -278,15 +283,15 @@ static void mix_redraw_ce (REMAILER **type2_list,
 {
   if (!coords || !chain)
     return;
-  
+
   if (coords[i].r < MIX_MAXROW)
   {
-    
+
     if (selected)
       SETCOLOR (MT_COLOR_INDICATOR);
     else
       NORMAL_COLOR;
-    
+
     mutt_window_mvaddstr (MuttIndexWindow, coords[i].r, coords[i].c,
                           type2_list[chain->ch[i]]->shortname);
     NORMAL_COLOR;
@@ -302,7 +307,7 @@ static void mix_redraw_chain (REMAILER **type2_list,
 			      int cur)
 {
   int i;
-  
+
   for (i = MIX_VOFFSET; i < MIX_MAXROW; i++)
   {
     mutt_window_move (MuttIndexWindow, i, 0);
@@ -326,17 +331,17 @@ static const char *mix_format_caps (REMAILER *r)
 {
   static char capbuff[10];
   char *t = capbuff;
-  
+
   if (r->caps & MIX_CAP_COMPRESS)
     *t++ = 'C';
   else
     *t++ = ' ';
-  
+
   if (r->caps & MIX_CAP_MIDDLEMAN)
     *t++ = 'M';
   else
     *t++ = ' ';
-  
+
   if (r->caps & MIX_CAP_NEWSPOST)
   {
     *t++ = 'N';
@@ -347,7 +352,7 @@ static const char *mix_format_caps (REMAILER *r)
     *t++ = ' ';
     *t++ = ' ';
   }
-   
+
   if (r->caps & MIX_CAP_NEWSMAIL)
   {
     *t++ = 'N';
@@ -358,15 +363,15 @@ static const char *mix_format_caps (REMAILER *r)
     *t++ = ' ';
     *t++ = ' ';
   }
-  
+
   *t = '\0';
-  
+
   return capbuff;
 }
 
 /*
  * Format an entry for the remailer menu.
- * 
+ *
  * %n	number
  * %c	capabilities
  * %s	short name
@@ -424,7 +429,7 @@ static const char *mix_entry_fmt (char *dest,
       else if (!remailer->addr)
         optional = 0;
       break;
-    
+
     default:
       *dest = '\0';
   }
@@ -437,7 +442,7 @@ static const char *mix_entry_fmt (char *dest,
 }
 
 
-  
+
 static void mix_entry (char *b, size_t blen, MUTTMENU *menu, int num)
 {
   REMAILER **type2_list = (REMAILER **) menu->data;
@@ -445,14 +450,14 @@ static void mix_entry (char *b, size_t blen, MUTTMENU *menu, int num)
 		     (unsigned long) type2_list[num], MUTT_FORMAT_ARROWCURSOR);
 }
 
-static int mix_chain_add (MIXCHAIN *chain, const char *s, 
+static int mix_chain_add (MIXCHAIN *chain, const char *s,
 			  REMAILER **type2_list)
 {
   int i;
-  
+
   if (chain->cl >= MAXMIXES)
     return -1;
-  
+
   if (!mutt_strcmp (s, "0") || !ascii_strcasecmp (s, "<random>"))
   {
     chain->ch[chain->cl++] = 0;
@@ -467,16 +472,16 @@ static int mix_chain_add (MIXCHAIN *chain, const char *s,
       return 0;
     }
   }
-  
+
   /* replace unknown remailers by <random> */
-  
+
   if (!type2_list[i])
     chain->ch[chain->cl++] = 0;
 
   return 0;
 }
 
-static const struct mapping_t RemailerHelp[] = 
+static const struct mapping_t RemailerHelp[] =
 {
   { N_("Append"), OP_MIX_APPEND },
   { N_("Insert"), OP_MIX_INSERT },
@@ -485,7 +490,7 @@ static const struct mapping_t RemailerHelp[] =
   { N_("OK"),     OP_MIX_USE    },
   { NULL,         0 }
 };
-  
+
 
 void mix_make_chain (LIST **chainp)
 {
@@ -493,17 +498,17 @@ void mix_make_chain (LIST **chainp)
   MIXCHAIN *chain;
   int c_cur = 0, c_old = 0;
   short c_redraw = 1;
-  
+
   REMAILER **type2_list = NULL;
   size_t ttll = 0;
-  
+
   struct coord *coords = NULL;
-  
+
   MUTTMENU *menu;
   char helpstr[LONG_STRING];
   short loop = 1;
   int op;
-  
+
   int i, j;
   char *t;
 
@@ -518,16 +523,16 @@ void mix_make_chain (LIST **chainp)
     mix_chain_add (chain, (char *) p->data, type2_list);
 
   mutt_free_list (chainp);
-  
+
   /* safety check */
   for (i = 0; i < chain->cl; i++)
   {
     if (chain->ch[i] >= ttll)
       chain->ch[i] = 0;
   }
-  
+
   mix_screen_coordinates (type2_list, &coords, chain, 0);
-  
+
   menu = mutt_new_menu (MENU_MIX);
   menu->max = ttll;
   menu->make_entry = mix_entry;
@@ -537,15 +542,15 @@ void mix_make_chain (LIST **chainp)
   menu->help = mutt_compile_help (helpstr, sizeof (helpstr), MENU_MIX, RemailerHelp);
   menu->pagelen = MIX_VOFFSET - 1;
   mutt_push_current_menu (menu);
-  
-  while (loop) 
+
+  while (loop)
   {
     if (menu->pagelen != MIX_VOFFSET - 1)
     {
       menu->pagelen = MIX_VOFFSET - 1;
       menu->redraw = REDRAW_FULL;
     }
-    
+
     if (c_redraw)
     {
       mix_redraw_head (chain);
@@ -557,9 +562,9 @@ void mix_make_chain (LIST **chainp)
       mix_redraw_ce (type2_list, coords, chain, c_old, 0);
       mix_redraw_ce (type2_list, coords, chain, c_cur, 1);
     }
-    
+
     c_old = c_cur;
-    
+
     switch ((op = mutt_menuLoop (menu)))
     {
       case OP_REDRAW:
@@ -571,7 +576,7 @@ void mix_make_chain (LIST **chainp)
 	menu->pagelen = MIX_VOFFSET - 1;
 	break;
       }
-      
+
       case OP_EXIT:
       {
 	chain->cl = 0;
@@ -588,12 +593,12 @@ void mix_make_chain (LIST **chainp)
 	  mix_screen_coordinates (type2_list, &coords, chain, c_cur);
 	  c_redraw = 1;
 	}
-	
-	if (chain->cl && chain->ch[chain->cl - 1] && 
+
+	if (chain->cl && chain->ch[chain->cl - 1] &&
 	    (type2_list[chain->ch[chain->cl-1]]->caps & MIX_CAP_MIDDLEMAN))
 	{
 	  mutt_error ( _("Error: %s can't be used as the final remailer of a chain."),
-		    type2_list[chain->ch[chain->cl - 1]]->shortname);
+                       type2_list[chain->ch[chain->cl - 1]]->shortname);
 	}
 	else
 	{
@@ -616,30 +621,30 @@ void mix_make_chain (LIST **chainp)
 	  chain->cl++;
 	  for (i = chain->cl - 1; i > c_cur; i--)
 	    chain->ch[i] = chain->ch[i-1];
-	  
+
 	  chain->ch[c_cur] = menu->current;
 	  mix_screen_coordinates (type2_list, &coords, chain, c_cur);
 	  c_redraw = 1;
 	}
 	else
 	  mutt_error ( _("Mixmaster chains are limited to %d elements."),
-		    MAXMIXES);
-	
+                       MAXMIXES);
+
 	break;
       }
-      
+
       case OP_MIX_DELETE:
       {
 	if (chain->cl)
 	{
 	  chain->cl--;
-	  
+
 	  for (i = c_cur; i < chain->cl; i++)
 	    chain->ch[i] = chain->ch[i+1];
 
 	  if (c_cur == chain->cl && c_cur)
 	    c_cur--;
-	  
+
 	  mix_screen_coordinates (type2_list, &coords, chain, c_cur);
 	  c_redraw = 1;
 	}
@@ -649,34 +654,34 @@ void mix_make_chain (LIST **chainp)
 	}
 	break;
       }
-      
+
       case OP_MIX_CHAIN_PREV:
       {
-	if (c_cur) 
+	if (c_cur)
 	  c_cur--;
 	else
 	  mutt_error _("You already have the first chain element selected.");
-	
+
 	break;
       }
-      
+
       case OP_MIX_CHAIN_NEXT:
       {
 	if (chain->cl && c_cur < chain->cl - 1)
 	  c_cur++;
 	else
 	  mutt_error _("You already have the last chain element selected.");
-	
+
 	break;
       }
     }
   }
-  
+
   mutt_pop_current_menu (menu);
   mutt_menuDestroy (&menu);
 
   /* construct the remailer list */
-  
+
   if (chain->cl)
   {
     for (i = 0; i < chain->cl; i++)
@@ -685,11 +690,11 @@ void mix_make_chain (LIST **chainp)
 	t = type2_list[j]->shortname;
       else
 	t = "*";
-      
+
       *chainp = mutt_add_list (*chainp, t);
     }
   }
-  
+
   mix_free_type2_list (&type2_list);
   FREE (&coords);
   FREE (&chain);
@@ -702,7 +707,7 @@ int mix_check_message (HEADER *msg)
   const char *fqdn;
   short need_hostname = 0;
   ADDRESS *p;
-  
+
   if (msg->env->cc || msg->env->bcc)
   {
     mutt_error _("Mixmaster doesn't accept Cc or Bcc headers.");
@@ -711,7 +716,7 @@ int mix_check_message (HEADER *msg)
 
   /* When using mixmaster, we MUST qualify any addresses since
    * the message will be delivered through remote systems.
-   * 
+   *
    * use_domain won't be respected at this point, hidden_host will.
    */
 
@@ -723,16 +728,16 @@ int mix_check_message (HEADER *msg)
       break;
     }
   }
-    
+
   if (need_hostname)
   {
-    
+
     if (!(fqdn = mutt_fqdn (1)))
     {
       mutt_error _("Please set the hostname variable to a proper value when using mixmaster!");
       return (-1);
     }
-  
+
     /* Cc and Bcc are empty at this point. */
     rfc822_qualify (msg->env->to, fqdn);
     rfc822_qualify (msg->env->reply_to, fqdn);
@@ -745,17 +750,19 @@ int mix_check_message (HEADER *msg)
 int mix_send_message (LIST *chain, const char *tempfile)
 {
   BUFFER *cmd;
-  char cd_quoted[STRING];
+  BUFFER *cd_quoted;
   int i;
 
   cmd = mutt_buffer_pool_get ();
+  cd_quoted = mutt_buffer_pool_get ();
+
   mutt_buffer_printf (cmd, "cat %s | %s -m ", tempfile, Mixmaster);
 
   for (i = 0; chain; chain = chain->next, i = 1)
   {
     mutt_buffer_addstr (cmd, i ? "," : " -l ");
-    mutt_quote_filename (cd_quoted, sizeof (cd_quoted), (char *) chain->data);
-    mutt_buffer_addstr (cmd, cd_quoted);
+    mutt_buffer_quote_filename (cd_quoted, (char *) chain->data);
+    mutt_buffer_addstr (cmd, mutt_b2s (cd_quoted));
   }
 
   if (!option (OPTNOCURSES))
@@ -772,6 +779,7 @@ int mix_send_message (LIST *chain, const char *tempfile)
   }
 
   mutt_buffer_pool_release (&cmd);
+  mutt_buffer_pool_release (&cd_quoted);
   unlink (tempfile);
   return i;
 }
