@@ -1,21 +1,21 @@
 /*
  * Copyright (C) 1996-2000 Michael R. Elkins <me@mutt.org>
  * Copyright (C) 2011-2013 Michael R. Elkins <me@mutt.org>
- * 
+ *
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
  *     the Free Software Foundation; either version 2 of the License, or
  *     (at your option) any later version.
- * 
+ *
  *     This program is distributed in the hope that it will be useful,
  *     but WITHOUT ANY WARRANTY; without even the implied warranty of
  *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *     GNU General Public License for more details.
- * 
+ *
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
  *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
- */ 
+ */
 
 #if HAVE_CONFIG_H
 # include "config.h"
@@ -37,8 +37,14 @@
 
 #include "mutt_idna.h"
 
-#define terminate_string(a, b, c) do { if ((b) < (c)) a[(b)] = 0; else \
-	a[(c)] = 0; } while (0)
+#define terminate_string(a, b, c)               \
+  do                                            \
+  {                                             \
+    if ((b) < (c))                              \
+      a[(b)] = 0;                               \
+    else                                        \
+      a[(c)] = 0;                               \
+  } while (0)
 
 #define terminate_buffer(a, b) terminate_string(a, b, sizeof (a) - 1)
 
@@ -141,7 +147,7 @@ parse_comment (const char *s,
 	       char *comment, size_t *commentlen, size_t commentmax)
 {
   int level = 1;
-  
+
   while (*s && level)
   {
     if (*s == '(')
@@ -356,7 +362,7 @@ add_addrspec (ADDRESS **top, ADDRESS **last, const char *phrase,
 	      char *comment, size_t *commentlen, size_t commentmax)
 {
   ADDRESS *cur = rfc822_new_address ();
-  
+
   if (parse_addr_spec (phrase, comment, commentlen, commentmax, cur) == NULL)
   {
     rfc822_free_address (&cur);
@@ -380,7 +386,7 @@ ADDRESS *rfc822_parse_adrlist (ADDRESS *top, const char *s)
   char comment[LONG_STRING], phrase[LONG_STRING];
   size_t phraselen = 0, commentlen = 0;
   ADDRESS *cur, *last = NULL;
-  
+
   RFC822Error = 0;
 
   last = top;
@@ -390,7 +396,7 @@ ADDRESS *rfc822_parse_adrlist (ADDRESS *top, const char *s)
   ws_pending = is_email_wsp (*s);
   if ((nl = mutt_strlen (s)))
     nl = s[nl - 1] == '\n';
-  
+
   s = skip_email_wsp(s);
 #ifdef EXACT_ADDRESS
   begin = s;
@@ -494,10 +500,10 @@ ADDRESS *rfc822_parse_adrlist (ADDRESS *top, const char *s)
 
       phraselen = 0;
       commentlen = 0;
-      s++;
 #ifdef EXACT_ADDRESS
-      begin = skip_email_wsp(s);
+      begin = s;
 #endif
+      s++;
     }
     else if (*s == '<')
     {
@@ -536,7 +542,7 @@ ADDRESS *rfc822_parse_adrlist (ADDRESS *top, const char *s)
     ws_pending = is_email_wsp(*s);
     s = skip_email_wsp(s);
   }
-  
+
   if (phraselen)
   {
     terminate_buffer (phrase, phraselen);
@@ -603,7 +609,7 @@ void rfc822_write_address_single (char *buf, size_t buflen, ADDRESS *addr,
   size_t len;
   char *pbuf = buf;
   char *pc;
-  
+
   if (!addr)
     return;
 
@@ -739,7 +745,7 @@ int rfc822_write_address (char *buf, size_t buflen, ADDRESS *addr, int display)
 {
   char *pbuf = buf;
   size_t len = mutt_strlen (buf);
-  
+
   buflen--; /* save room for the terminal nul */
 
   if (len > 0)
@@ -810,7 +816,7 @@ ADDRESS *rfc822_cpy_adr_real (ADDRESS *addr)
 ADDRESS *rfc822_cpy_adr (ADDRESS *addr, int prune)
 {
   ADDRESS *top = NULL, *last = NULL;
-  
+
   for (; addr; addr = addr->next)
   {
     if (prune && addr->group && (!addr->next || !addr->next->mailbox))
@@ -907,7 +913,7 @@ int main (int argc, char **argv)
 # else
   char *str = "a b c ";
 # endif
-  
+
   list = rfc822_parse_adrlist (NULL, str);
   buf[0] = 0;
   rfc822_write_address (buf, sizeof (buf), list);
