@@ -1,20 +1,20 @@
 /*
  * Copyright (C) 1996-2000 Michael R. Elkins <me@mutt.org>
- * 
+ *
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
  *     the Free Software Foundation; either version 2 of the License, or
  *     (at your option) any later version.
- * 
+ *
  *     This program is distributed in the hope that it will be useful,
  *     but WITHOUT ANY WARRANTY; without even the implied warranty of
  *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *     GNU General Public License for more details.
- * 
+ *
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
  *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
- */ 
+ */
 
 #if HAVE_CONFIG_H
 # include "config.h"
@@ -34,15 +34,17 @@
 /* function to use as discriminator when normal sort method is equal */
 static sort_t *AuxSort = NULL;
 
-#define AUXSORT(code,a,b) if (!code && AuxSort && !option(OPTAUXSORT)) { \
-  set_option(OPTAUXSORT); \
-  code = AuxSort(a,b); \
-  unset_option(OPTAUXSORT); \
-  if (code) \
-    return (code); \
-} \
-if (!code) \
-  code = (*((HEADER **)a))->index - (*((HEADER **)b))->index;
+#define AUXSORT(code,a,b)                                       \
+  if (!code && AuxSort && !option(OPTAUXSORT))                  \
+  {                                                             \
+    set_option(OPTAUXSORT);                                     \
+    code = AuxSort(a,b);                                        \
+    unset_option(OPTAUXSORT);                                   \
+    if (code)                                                   \
+      return (code);                                            \
+  }                                                             \
+  if (!code)                                                    \
+    code = (*((HEADER **)a))->index - (*((HEADER **)b))->index;
 
 static int compare_score (const void *a, const void *b)
 {
@@ -277,7 +279,7 @@ void mutt_sort_headers (CONTEXT *ctx, int init)
   HEADER *h;
   THREAD *thread, *top;
   sort_t *sortfunc;
-  
+
   unset_option (OPTNEEDRESORT);
 
   if (!ctx)
@@ -337,7 +339,7 @@ void mutt_sort_headers (CONTEXT *ctx, int init)
     mutt_sleep (1);
     return;
   }
-  else 
+  else
     qsort ((void *) ctx->hdrs, ctx->msgcount, sizeof (HEADER *), sortfunc);
 
   /* adjust the virtual message numbers */
