@@ -1,31 +1,33 @@
 /*
  * Copyright (C) 1996-2000,2007 Michael R. Elkins <me@mutt.org>
- * 
+ *
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
  *     the Free Software Foundation; either version 2 of the License, or
  *     (at your option) any later version.
- * 
+ *
  *     This program is distributed in the hope that it will be useful,
  *     but WITHOUT ANY WARRANTY; without even the implied warranty of
  *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *     GNU General Public License for more details.
- * 
+ *
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
  *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
- */ 
+ */
 
 #if HAVE_CONFIG_H
 # include "config.h"
 #endif
 
+#include "version.h"
 #include "mutt.h"
 #include "mutt_menu.h"
 #include "mutt_curses.h"
 #include "sort.h"
 #include "mapping.h"
 #include "mx.h"
+#include "buffy.h"
 
 #include <string.h>
 #include <ctype.h>
@@ -55,11 +57,12 @@ static void _menu_status_line (char *buf, size_t buflen, size_t col, int cols, M
  * %p = number of postponed messages [option]
  * %P = percent of way through index
  * %r = readonly/wontwrite/changed flag
+ * %R = number of read messages [option]
  * %s = current sorting method ($sort)
  * %S = current aux sorting method ($sort_aux)
  * %t = # of tagged messages [option]
  * %u = number of unread messages [option]
- * %v = Mutt version 
+ * %v = Mutt version
  * %V = currently active limit pattern [option] */
 static const char *
 status_format_str (char *buf, size_t buflen, size_t col, int cols, char op, const char *src,
@@ -97,18 +100,20 @@ status_format_str (char *buf, size_t buflen, size_t col, int cols, char op, cons
     case 'f':
       snprintf (fmt, sizeof(fmt), "%%%ss", prefix);
 #ifdef USE_COMPRESSED
-      if (Context && Context->compress_info && Context->realpath) {
-	 strfcpy (tmp, Context->realpath, sizeof (tmp));
-	 mutt_pretty_mailbox (tmp, sizeof (tmp));
-      } else
-#endif
-      if (Context && Context->path)
+      if (Context && Context->compress_info && Context->realpath)
       {
-	strfcpy (tmp, Context->path, sizeof (tmp));
-	mutt_pretty_mailbox (tmp, sizeof (tmp));
+        strfcpy (tmp, Context->realpath, sizeof (tmp));
+        mutt_pretty_mailbox (tmp, sizeof (tmp));
       }
       else
-	strfcpy (tmp, _("(no mailbox)"), sizeof (tmp));
+#endif
+        if (Context && Context->path)
+        {
+          strfcpy (tmp, Context->path, sizeof (tmp));
+          mutt_pretty_mailbox (tmp, sizeof (tmp));
+        }
+        else
+          strfcpy (tmp, _("(no mailbox)"), sizeof (tmp));
       snprintf (buf, buflen, fmt, tmp);
       break;
 
@@ -227,7 +232,7 @@ status_format_str (char *buf, size_t buflen, size_t col, int cols, char op, cons
           (Context->magic != MUTT_IMAP &&
            Context->deleted)) ? 1 : 0);
       }
-      
+
       if (!StChars || !StChars->len)
 	buf[0] = 0;
       else if (i >= StChars->len)
@@ -236,7 +241,21 @@ status_format_str (char *buf, size_t buflen, size_t col, int cols, char op, cons
         snprintf (buf, buflen, "%s", StChars->chars[i]);
       break;
     }
-      
+
+    case 'R':
+    {
+      int read = Context ? Context->msgcount - Context->unread : 0;
+
+      if (!optional)
+      {
+	snprintf (fmt, sizeof (fmt), "%%%sd", prefix);
+	snprintf (buf, buflen, fmt, read);
+      }
+      else if (!read)
+	optional = 0;
+      break;
+    }
+
     case 's':
       snprintf (fmt, sizeof (fmt), "%%%ss", prefix);
       snprintf (buf, buflen, fmt,
