@@ -5,16 +5,16 @@
  *     it under the terms of the GNU General Public License as published by
  *     the Free Software Foundation; either version 2 of the License, or
  *     (at your option) any later version.
- * 
+ *
  *     This program is distributed in the hope that it will be useful,
  *     but WITHOUT ANY WARRANTY; without even the implied warranty of
  *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *     GNU General Public License for more details.
- * 
+ *
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
  *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
- */ 
+ */
 
 #if HAVE_CONFIG_H
 # include "config.h"
@@ -22,6 +22,7 @@
 
 #include "mutt.h"
 #include "sort.h"
+#include "mailbox.h"
 
 #include <string.h>
 #include <ctype.h>
@@ -66,7 +67,7 @@ static int need_display_subject (CONTEXT *ctx, HEADER *hdr)
 	break;
     }
   }
-  
+
   /* if there is a parent-to-child subject change anywhere between us and our
    * closest displayed ancestor, display the subject */
   for (tmp = tree->parent; tmp; tmp = tmp->parent)
@@ -196,14 +197,14 @@ static void calculate_visibility (CONTEXT *ctx, int *max_depth)
 	tree = tree->prev;
     }
   }
-  
+
   /* now fix up for the OPTHIDETOP* options if necessary */
   if (hide_top_limited || hide_top_missing)
   {
     tree = ctx->tree;
     FOREVER
     {
-      if (!tree->visible && tree->deep && tree->subtree_visible < 2 
+      if (!tree->visible && tree->deep && tree->subtree_visible < 2
 	  && ((tree->message && hide_top_limited) || (!tree->message && hide_top_missing)))
 	tree->deep = 0;
       if (!tree->deep && tree->child && tree->subtree_visible)
@@ -260,7 +261,7 @@ void mutt_draw_tree (CONTEXT *ctx)
 	myarrow[0] = vtee;
       if (width == 2)
 	myarrow[1] = pseudo ?  MUTT_TREE_STAR
-	                     : (tree->duplicate_thread ? MUTT_TREE_EQUALS : MUTT_TREE_HLINE);
+          : (tree->duplicate_thread ? MUTT_TREE_EQUALS : MUTT_TREE_HLINE);
       if (tree->visible)
       {
 	myarrow[width] = MUTT_TREE_RARROW;
@@ -354,7 +355,7 @@ static LIST *make_subject_list (THREAD *cur, time_t *dateptr)
   time_t thisdate;
   LIST *curlist, *oldlist, *newlist, *subjects = NULL;
   int rc = 0;
-  
+
   FOREVER
   {
     while (!cur->message)
@@ -408,25 +409,22 @@ static LIST *make_subject_list (THREAD *cur, time_t *dateptr)
   return (subjects);
 }
 
-/* find the best possible match for a parent mesage based upon subject.
+/* find the best possible match for a parent message based upon subject.
  * if there are multiple matches, the one which was sent the latest, but
- * before the current message, is used. 
+ * before the current message, is used.
  */
 static THREAD *find_subject (CONTEXT *ctx, THREAD *cur)
 {
   struct hash_elem *ptr;
   THREAD *tmp, *last = NULL;
-  unsigned int hash;
   LIST *subjects = NULL, *oldlist;
-  time_t date = 0;  
+  time_t date = 0;
 
   subjects = make_subject_list (cur, &date);
 
   while (subjects)
   {
-    hash = ctx->subj_hash->hash_string ((unsigned char *) subjects->data,
-					ctx->subj_hash->nelem);
-    for (ptr = ctx->subj_hash->table[hash]; ptr; ptr = ptr->next)
+    for (ptr = hash_find_bucket (ctx->subj_hash, subjects->data); ptr; ptr = ptr->next)
     {
       tmp = ((HEADER *) ptr->data)->thread;
       if (tmp != cur &&			   /* don't match the same message */
@@ -590,9 +588,9 @@ THREAD *mutt_sort_subthreads (THREAD *thread, int init)
   THREAD **array, *sort_key, *top, *tmp;
   HEADER *oldsort_key;
   int i, array_size, sort_top = 0;
-  
+
   /* we put things into the array backwards to save some cycles,
-   * but we want to have to move less stuff around if we're 
+   * but we want to have to move less stuff around if we're
    * resorting, so we sort backwards and then put them back
    * in reverse order so they're forwards
    */
@@ -753,19 +751,19 @@ void mutt_sort_threads (CONTEXT *ctx, int init)
   int i, oldsort, using_refs = 0;
   THREAD *thread, *new, *tmp, top;
   LIST *ref = NULL;
-  
+
   /* set Sort to the secondary method to support the set sort_aux=reverse-*
    * settings.  The sorting functions just look at the value of
    * SORT_REVERSE
    */
   oldsort = Sort;
   Sort = SortAux;
-  
+
   if (!ctx->thread_hash)
     init = 1;
 
   if (init)
-    ctx->thread_hash = hash_create (ctx->msgcount * 2, 0);
+    ctx->thread_hash = hash_create (ctx->msgcount * 2, MUTT_HASH_ALLOW_DUPS);
 
   /* we want a quick way to see if things are actually attached to the top of the
    * thread tree or if they're just dangling, so we attach everything to a top
@@ -837,7 +835,7 @@ void mutt_sort_threads (CONTEXT *ctx, int init)
 	cur->thread = thread;
 	hash_insert (ctx->thread_hash,
 		     cur->env->message_id ? cur->env->message_id : "",
-		     thread, 1);
+		     thread);
 
 	if (new)
 	{
@@ -911,20 +909,20 @@ void mutt_sort_threads (CONTEXT *ctx, int init)
 	    ref = cur->env->references;
 	  else
 	    ref = cur->env->references->next;
-	  
+
 	  using_refs = 2;
 	}
       }
       else
 	ref = ref->next; /* go on with references */
-      
+
       if (!ref)
 	break;
 
       if ((new = hash_find (ctx->thread_hash, ref->data)) == NULL)
       {
 	new = safe_calloc (1, sizeof (THREAD));
-	hash_insert (ctx->thread_hash, ref->data, new, 1);
+	hash_insert (ctx->thread_hash, ref->data, new);
       }
       else
       {
@@ -964,7 +962,7 @@ void mutt_sort_threads (CONTEXT *ctx, int init)
 
     /* restore the oldsort order. */
     Sort = oldsort;
-    
+
     /* Put the list into an array. */
     linearize_tree (ctx);
 
@@ -1053,7 +1051,7 @@ int _mutt_aside_thread (HEADER *hdr, short dir, short subthreads)
   if ((dir != 0) ^ ((Sort & SORT_REVERSE) != 0))
   {
     do
-    { 
+    {
       cur = cur->next;
       if (!cur)
 	return (-1);
@@ -1063,7 +1061,7 @@ int _mutt_aside_thread (HEADER *hdr, short dir, short subthreads)
   else
   {
     do
-    { 
+    {
       cur = cur->prev;
       if (!cur)
 	return (-1);
@@ -1074,9 +1072,10 @@ int _mutt_aside_thread (HEADER *hdr, short dir, short subthreads)
   return (tmp->virtual);
 }
 
-int mutt_parent_message (CONTEXT *ctx, HEADER *hdr)
+int mutt_parent_message (CONTEXT *ctx, HEADER *hdr, int find_root)
 {
   THREAD *thread;
+  HEADER *parent = NULL;
 
   if ((Sort & SORT_MASK) != SORT_THREADS)
   {
@@ -1084,31 +1083,44 @@ int mutt_parent_message (CONTEXT *ctx, HEADER *hdr)
     return (hdr->virtual);
   }
 
+  /* Root may be the current message */
+  if (find_root)
+    parent = hdr;
+
   for (thread = hdr->thread->parent; thread; thread = thread->parent)
   {
     if ((hdr = thread->message) != NULL)
     {
-      if (VISIBLE (hdr, ctx))
-	return (hdr->virtual);
-      else
-      {
-	mutt_error _("Parent message is not visible in this limited view.");
-	return (-1);
-      }
+      parent = hdr;
+      if (!find_root)
+        break;
     }
   }
-  
-  mutt_error _("Parent message is not available.");
-  return (-1);
+
+  if (!parent)
+  {
+    mutt_error _("Parent message is not available.");
+    return (-1);
+  }
+  if (!VISIBLE (parent, ctx))
+  {
+    if (find_root)
+      mutt_error _("Root message is not visible in this limited view.");
+    else
+      mutt_error _("Parent message is not visible in this limited view.");
+    return (-1);
+  }
+  return (parent->virtual);
 }
 
 void mutt_set_virtual (CONTEXT *ctx)
 {
-  int i;
+  int i, padding;
   HEADER *cur;
 
   ctx->vcount = 0;
   ctx->vsize = 0;
+  padding = mx_msg_padding_size (ctx);
 
   for (i = 0; i < ctx->msgcount; i++)
   {
@@ -1118,7 +1130,8 @@ void mutt_set_virtual (CONTEXT *ctx)
       cur->virtual = ctx->vcount;
       ctx->v2r[ctx->vcount] = i;
       ctx->vcount++;
-      ctx->vsize += cur->content->length + cur->content->offset - cur->content->hdr_offset;
+      ctx->vsize += cur->content->length + cur->content->offset -
+        cur->content->hdr_offset + padding;
       cur->num_hidden = mutt_get_hidden (ctx, cur);
     }
   }
@@ -1189,7 +1202,7 @@ int _mutt_traverse_thread (CONTEXT *ctx, HEADER *cur, int flag)
     else if (flag & MUTT_THREAD_NEXT_UNREAD)
       return (min_unread);
   }
-  
+
   FOREVER
   {
     cur = thread->message;
@@ -1218,7 +1231,7 @@ int _mutt_traverse_thread (CONTEXT *ctx, HEADER *cur, int flag)
 	  if (cur != roothdr)
 	    cur->virtual = -1;
 	}
-	else 
+	else
 	{
 	  if (CHECK_LIMIT)
 	    cur->virtual = cur->msgno;
@@ -1301,16 +1314,16 @@ int mutt_messages_in_thread (CONTEXT *ctx, HEADER *hdr, int flag)
   {
     while (!threads[i]->message)
       threads[i] = threads[i]->child;
-  } 
+  }
 
   if (Sort & SORT_REVERSE)
     rc = threads[0]->message->msgno - (threads[1] ? threads[1]->message->msgno : -1);
   else
     rc = (threads[1] ? threads[1]->message->msgno : ctx->msgcount) - threads[0]->message->msgno;
-  
+
   if (flag)
     rc += 1;
-  
+
   return (rc);
 }
 
@@ -1327,7 +1340,7 @@ HASH *mutt_make_id_hash (CONTEXT *ctx)
   {
     hdr = ctx->hdrs[i];
     if (hdr->env->message_id)
-      hash_insert (hash, hdr->env->message_id, hdr, 0);
+      hash_insert (hash, hdr->env->message_id, hdr);
   }
 
   return hash;
@@ -1339,13 +1352,13 @@ HASH *mutt_make_subj_hash (CONTEXT *ctx)
   HEADER *hdr;
   HASH *hash;
 
-  hash = hash_create (ctx->msgcount * 2, 0);
+  hash = hash_create (ctx->msgcount * 2, MUTT_HASH_ALLOW_DUPS);
 
   for (i = 0; i < ctx->msgcount; i++)
   {
     hdr = ctx->hdrs[i];
     if (hdr->env->real_subj)
-      hash_insert (hash, hdr->env->real_subj, hdr, 1);
+      hash_insert (hash, hdr->env->real_subj, hdr);
   }
 
   return hash;
@@ -1383,7 +1396,8 @@ static void clean_references (THREAD *brk, THREAD *cur)
       /* clearing the References: header from obsolete Message-ID(s) */
       mutt_free_list (&ref->next);
 
-      h->env->refs_changed = h->changed = 1;
+      h->changed = 1;
+      h->env->changed |= MUTT_ENV_CHANGED_REFS;
     }
   }
 }
@@ -1392,7 +1406,8 @@ void mutt_break_thread (HEADER *hdr)
 {
   mutt_free_list (&hdr->env->in_reply_to);
   mutt_free_list (&hdr->env->references);
-  hdr->env->irt_changed = hdr->env->refs_changed = hdr->changed = 1;
+  hdr->changed = 1;
+  hdr->env->changed |= (MUTT_ENV_CHANGED_IRT | MUTT_ENV_CHANGED_REFS);
 
   clean_references (hdr->thread, hdr->thread->child);
 }
@@ -1406,10 +1421,11 @@ static int link_threads (HEADER *parent, HEADER *child, CONTEXT *ctx)
 
   child->env->in_reply_to = mutt_new_list ();
   child->env->in_reply_to->data = safe_strdup (parent->env->message_id);
-  
+
   mutt_set_flag (ctx, child, MUTT_TAG, 0);
-  
-  child->env->irt_changed = child->changed = 1;
+
+  child->changed = 1;
+  child->env->changed |= MUTT_ENV_CHANGED_IRT;
   return 1;
 }
 
